//------------------ VERSION MAIN 1 -------------------\\
//---------- STARTED JULY 23, 2012 11:30 PM -----------\\
//------------- ARDUINO LEONARDO 5V 16MHZ -------------\\

//--------------------------------------\\
//------------- LIBRARIES --------------\\

#include <EasyTransfer.h>
#include <LiquidCrystal.h>

//--------------------------------------\\
//------------ EASYTRANSFER ------------\\

EasyTransfer ETSerial; // Object used for EasyTransfer serial communication

struct RECEIVE_DATA_STRUCTURE // Set of data used in EasyTransfer communication
{
  int xAng;
  int yAng;
  int zAng;
  int xDeg;
  int yDeg;
  //int zDeg;
  float rX;
  float rY;
  //float rZ;
  float rRadius;
  int rDegree;
  int sendSpare;
};

RECEIVE_DATA_STRUCTURE accelData; // Name of data structure used in EasyTransfer communication

//--------------------------------------\\
//---------------- LCD -----------------\\

LiquidCrystal lcd(7, 6, 5, 4, 3, 2); // Define LCD IO pins

//--------------------------------------\\
//-------------- VARIABLES -------------\\

  // Define all motor driver pins
const int motorB2 = 10; // Left motor
const int motorB1 = 8;
const int motorA2 = 12; // Right motor
const int motorA1 = 11;
const int motorAPwm = 9; // Right motor 
const int motorBPwm = 13; // Left motor

//int rightMotorSpeed; // PWM speed of motors
//int leftMotorSpeed;

int motorSpeedMax = 240; // Maximum PWM value for motors
int motorSpeedMin = 160; // Minimum PWM value for motors

int turningMotor; // PWM pin of the motor that is allowing turning
int drivingMotor; // PWM pin of the mtor that is not allowing turning

int turningMotorSpeed; // Speed of the slower motor that is allowing turning
int drivingMotorSpeed; // Speed of regular speed motor that is not allowing turning

int turningMotorSpeedPwm; // Speed of the slower motor that is allowing turning in PWM
int drivingMotorSpeedPwm; // Speed of regular speed motor that is not allowing turning in PWM

int rightTilt; // Represents the amount of tilt in a certain direction
int leftTilt;
int forwardTilt;
int backwardTilt;

int xAngAbs; // The absolute value of xAng, positive 
int yAngAbs; // The absolute value of yAng, positive

int xAngMap; // Mapped xAngAbs
int yAngMap; // Mapped yAngAbs

int xAngMapMin = 1;
int xAngMapMax = 10;
int yAngMapMin = 1;
int yAngMapMax = 10;

int xAngAbsMax = 70; // Max & min values for AngAbs, used for map() function
int xAngAbsMin = 10;
int yAngAbsMax = 70;
int yAngAbsMin = 10;

/*boolean tiltForward; // Represent whether tilting forward or backward
boolean tiltBackward;
boolean tiltRight; // Represent whether tilting right or left
boolean tiltLeft;*/
boolean displayLCD;


//--------------------------------------\\
//--------------- SETUP ----------------\\

void setup()
{
  Serial.begin(19200); // Begin serial communication with computer for debugging
  Serial1.begin(9600); // Begin serial communication with XBee
  ETSerial.begin(details(accelData), &Serial1); // Intitialize EasyTransfer serial communication
  lcd.begin(16, 2); // Define LCD dimensions, initialize
  
  pinMode(motorA1, OUTPUT); // Set all motor pins to OUTPUTS
  pinMode(motorA2, OUTPUT); 
  pinMode(motorB1, OUTPUT);
  pinMode(motorB2, OUTPUT);
  pinMode(motorAPwm, OUTPUT);
  pinMode(motorBPwm, OUTPUT);
}

//--------------------------------------\\
//------------- MAIN LOOP --------------\\

void loop()
{
  if(ETSerial.receiveData())
  {
    
  //--------------------------------------\\
  //--------- LOCAL SERIAL DEBUG ---------\\
    
  Serial.print("xAng= "); Serial.print(accelData.xAng);
  Serial.print("   yAng= "); Serial.print(accelData.yAng);
  Serial.print("   zAng= "); Serial.print(accelData.zAng);
  /*Serial.print("   xDeg= "); Serial.print(accelData.xDeg);
  Serial.print("   yDeg= "); Serial.print(accelData.yDeg);
  Serial.print("   rX= "); Serial.print(accelData.rX);
  Serial.print("   rY= "); Serial.print(accelData.rY);*/
  //Serial.print("   rRadius= "); Serial.print(accelData.rRadius);
  //Serial.print("   rDegree= "); Serial.print(accelData.rDegree);
  
  //--------------------------------------\\
  
  int xAngAbs = abs(accelData.xAng);
  int yAngAbs = abs(accelData.yAng);
  
  Serial.print("   xAngAbs="); Serial.print(xAngAbs);
  Serial.print("   yAngAbs="); Serial.print(yAngAbs);
    
  int xAngMap = map(xAngAbs, xAngAbsMin, xAngAbsMax, xAngMapMax, xAngMapMin);
  Serial.print("   xAngMap="); Serial.print(xAngMap);
    
  int yAngMap = map(yAngAbs, yAngAbsMin, yAngAbsMax, yAngMapMin, yAngMapMax);
  Serial.print("   yAngMap="); Serial.print(yAngMap);
    
  int turningMotorSpeed = (xAngMap * yAngMap);
  //Serial.print("   turningMotorSpeed="); Serial.print(turningMotorSpeed);
    
  int drivingMotorSpeed = (xAngMapMax * yAngMap);
  //Serial.print("   drivingMotorSpeed="); Serial.println(drivingMotorSpeed);
    
  int turningMotorSpeedPwm = map(turningMotorSpeed, 10, 100, (motorSpeedMin-60), (motorSpeedMax-20));
  Serial.print("   turningMotorSpeedPwm="); Serial.print(turningMotorSpeedPwm);
    
  int drivingMotorSpeedPwm = map(drivingMotorSpeed, 10, 100, motorSpeedMin, motorSpeedMax);
  Serial.print("   drivingMotorSpeedPwm="); Serial.println(drivingMotorSpeedPwm);
  
  if(accelData.yAng > 10)
  {
    motorAForward();
    motorBForward();
    displayLCD = true;
  }
  
  if(accelData.yAng < -10)
  {
    motorABackward();
    motorBBackward();
    displayLCD = true;
  }
  
  if(accelData.xAng > 10)
  {
    turningMotor = motorBPwm;
    drivingMotor = motorAPwm;
    displayLCD = true;
  }
  
  if(accelData.xAng < -10)
  {
    turningMotor = motorAPwm;
    drivingMotor = motorBPwm;
    displayLCD = true;
  }
  
  if(accelData.xAng < 10 && accelData.xAng > -10)
  {
    turningMotorSpeedPwm = drivingMotorSpeedPwm;
    displayLCD = true;
  }

  if( (accelData.xAng < 10 && accelData.xAng > -10) && (accelData.yAng < 10 && accelData.yAng > -10) )
  {
    motorABrake();
    motorBBrake();
    turningMotorSpeedPwm = 0;
    drivingMotorSpeedPwm = 0;
    displayLCD = false;
  }
    
  analogWrite(turningMotor, turningMotorSpeedPwm);
  analogWrite(drivingMotor, drivingMotorSpeedPwm);
  
  if(displayLCD == true)
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("drivingMotor="); lcd.print(drivingMotorSpeedPwm);
    lcd.setCursor(0, 1);
    lcd.print("turningMotor="); lcd.print(turningMotorSpeedPwm);
  }
  
  if(displayLCD == false)
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("drivingMotor="); lcd.print("0");
    lcd.setCursor(0, 1);
    lcd.print("turningMotor="); lcd.print("0");
  }

  
  }
  
  delay(30);  // * MONITOR THIS DELAY, VERY IMPORTANT, MAY CAUSE MALFUNCTION, HAS TO BE SHORTER THAN DELAY IN CONTROLLER CODE, IF MALFUNCITON INCREASE DELAY

}  
//--------------------------------------\\

void motorAForward()
{
  digitalWrite(motorA1, HIGH);
  digitalWrite(motorA2, LOW);
}

void motorABackward()
{
  digitalWrite(motorA1, LOW);
  digitalWrite(motorA2, HIGH);
}

void motorBForward()
{
  digitalWrite(motorB1, HIGH);
  digitalWrite(motorB2, LOW);
}

void motorBBackward()
{
  digitalWrite(motorB1, LOW);
  digitalWrite(motorB2, HIGH);
}

void motorABrake()
{
  digitalWrite(motorA1, LOW);
  digitalWrite(motorA2, LOW);
}

void motorBBrake()
{
  digitalWrite(motorB1, LOW);
  digitalWrite(motorB2, LOW);
}

